//Variable Frequency Electrosense
//ME333 Winter 2010
//Pill Gun Park, Tod Reynolds, Joshua Peng

/** INCLUDES ***************************************************/
#include "HardwareProfile.h"
#include <plib.h>
#include "dsplib_dsp.h"
#include "fftc.h"
#include "Compiler.h"
#include <stdlib.h>
#include <GenericTypeDefs.h>
#include "math.h"

/** Constants **************************************************/

#define TRUE 		1
#define FALSE		0

#define PIN_A2			LATAbits.LATA2  //FSYNC
#define PIN_A3			LATAbits.LATA3	//SCLK
#define PIN_A14			LATAbits.LATA14	//SDATA

#define INPUT_A9       	PORTAbits.RA9	 //Switch from an end of our tank
#define INPUT_A10       PORTAbits.RA10	 //Switch from an end of our tank

#define DESIRED_BAUDRATE    	(38400)      // The desired BaudRate 
#define SYSCLK	(80000000)
#define PBCLK  (SYSCLK)

// Define constants for PINS D1 - D4
#define CLK			LATDbits.LATD1   //PIN D1
#define DIR			LATDbits.LATD2   //PIN D2

#define Forward		0
#define Reverse		1

//Number of steps to move actuator 1/2 inch
#define step_size	300

//Number of half inchs want to move
#define step_count	6

// To modify the number of samples:
#define fftc fft16c512 //from fftc.h, for N = 256 use fft16c256, for N = 1024 use fft16c1024	
#define N 512	// Also change the log2N variable below!!

// Also change the timer interupt setup in initInterruptController!

#pragma config FPBDIV = DIV_1            //Sets PBCLK to SYSCLK

/** Function Declarations **************************************/

void initInterruptController();	//initiate the timer interrupt

void initUART2(int pbClk);		//initiate UART2 interrupt

void sendDataRS232(void);		//RS232 data transmit

void computeFFT(void);  		//sets up function to compute FFT
	
void initADC();				//initiate ADC

void Delayms();				

void ad9833_set_freq(int []);		//send the AD9833 16 bits

void generateWave(int [], int []);	//generate wave

void wave(int);

void dcalibrate(void);			//calibrate

/** Global Variables *******************************************/

int computeFFTflag = FALSE; //used to indicate that an fft computation has been requested over rs232

int sampleIndex = 0; //keeps track of where we're putting our ADC reading

/** establish variables for FFT calculation*/
// -----
// this has to be changed for different values of N
//int log2N = 8; // log2(256) = 8
//int log2N = 9; // log2(512) = 9
int log2N = 9; // log2(1024) = 10
// -----
#define BITS16 16

long int singleSidedFFT1[N];
long int singleSidedFFT2[N];
float phase1[N];
float phase2[N];

int16c sampleBuffer1[N]; 	//initialize buffer to collect samples
int16c sampleBuffer2[N];
int16c calcBuffer1[N]; 		// initialize buffer to hold old samples
int16c calcBuffer2[N];
int16c scratch1[N];
int16c scratch2[N];
int16c dout1[N];			//holds computed FFT until transmission
int16c dout2[N];

float location;
long int freqVector[N];
int samfreq = 0;
int SAMPLEFREQ = 0;
unsigned speed = 10;
int step = 0;
int step2 = 0;
int halfinch = 0;			//position of the motor
 int cf = 0;

 int lsb100 [BITS16] = {0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 1, 1, 1, 1, 1, 0 };	//100,  10100111110
 int lsb200 [BITS16] = {0, 1, 0, 0, 1, 0, 1, 0, 0, 1, 1, 1, 1, 1, 0, 0 };	//200,  101001111100
 int lsb300 [BITS16] = {0, 1, 0, 0, 1, 1, 1, 1, 1, 0, 1, 1, 1, 0, 1, 0 };   //300,  111110111010
 int lsb400 [BITS16] = {0, 1, 0, 1, 0, 1, 0, 0, 1, 1, 1, 1, 1, 0, 0, 1 };	//400,  1 0 1 0 0 1 1 1 1 1 0 0 1
 int lsb500 [BITS16] = {0, 1, 0, 1, 1, 0, 1, 0, 0, 0, 1, 1, 0, 1, 1, 1 };	//500,  1 1 0 1 0 0 0 1 1 0 1 1 1
 int lsb600 [BITS16] = {0, 1, 0, 1, 1, 1, 1, 1, 0, 1, 1, 1, 0, 1, 0, 1 };	//600,  1 1 1 1 1 0 1 1 1 0 1 0 1
 int lsb700 [BITS16] = {0, 1, 1, 0, 0, 1, 0, 0, 1, 0, 1, 1, 0, 0, 1, 1 };	//700,  1 0 0 1 0 0 1 0 1 1 0 0 1 1
 int lsb800 [BITS16] = {0, 1, 1, 0, 1, 0, 0, 1, 1, 1, 1, 1, 0, 0, 0, 1 };	//800,  1 0 1 0 0 1 1 1 1 1 0 0 0 1
 int lsb900 [BITS16] = {0, 1, 1, 0, 1, 1, 1, 1, 0, 0, 1, 0, 1, 1, 1, 1 };	//900,  1 0 1 1 1 1 0 0 1 0 1 1 1 1
  int lsb1k [BITS16] = {0, 1, 1, 1, 0, 1, 0, 0, 0, 1, 1, 0, 1, 1, 1, 0 };	//1k,   1 1 0 1 0 0 0 1 1 0 1 1 1 0
  int lsb2k [BITS16] = {0, 1, 1, 0, 1, 0, 0, 0, 1, 1, 0, 1, 1, 0, 1, 1 };	//2k,   1 1 0 1 0 0 0 1 1 0 1 1 0 1 1
  int lsb3k [BITS16] = {0, 1, 0, 1, 1, 1, 0, 1, 0, 1, 0, 0, 1, 0, 0, 1 };	//3k,   1 0 0 1 1 1 0 1 0 1 0 0 1 0 0 1
  int lsb4k [BITS16] = {0, 1, 0, 1, 0, 0, 0, 1, 1, 0, 1, 1, 0, 1 ,1, 1 };	//4k,   1 1 0 1 0 0 0 1 1 0 1 1 0 1 1 1
  int lsb5k [BITS16] = {0, 1, 0, 0, 0, 1, 1, 0, 0, 0, 1, 0, 0, 1, 0, 1 };	//5k,   1  0  0  0  0  0  1  1  0  0  0  1  0  0  1  0  1
  int lsb6k [BITS16] = {0, 1, 1, 1, 1, 0, 1, 0, 1, 0, 0, 1, 0, 0, 1, 0 };	//6k,   1  0  0  1  1  1  0  1  0  1  0  0  1  0  0  1  0
  int lsb7k [BITS16] = {0, 1, 1, 0, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0 };	//7k,   1  0  1  1  0  1  1  1  1  0  0  0  0  0  0  0  0
  int lsb8k [BITS16] = {0, 1, 1, 0, 0, 0, 1 ,1, 0, 1, 1, 0, 1, 1, 1, 0 };	//8k,   1  1  0  1  0  0  0  1  1  0  1  1  0  1  1  1  0
  int lsb9k [BITS16] = {0, 1, 0, 1, 0, 1, 1, 1, 1, 1, 0, 1, 1, 0, 1 ,1 };	//9k,   1  1  1  0  1  0  1  1  1  1  1  0  1  1  0  1  1
 int lsb10k [BITS16] = {0, 1, 0, 0, 1, 1, 0, 0, 0, 1, 0, 0, 1, 0, 1, 0 }; 	//10k,  1  0  0  0  0  0  1  1  0  0  0  1  0  0  1  0  1  0

    int msb [BITS16] = {0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 ,0, 0, 0 };
  int msb2k [BITS16] = {0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1 };
  int msb3k [BITS16] = {0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0 };
  int msb4k [BITS16] = {0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1 };
  int msb5k [BITS16] = {0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0 };
  int msb6k [BITS16] = {0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0 };
  int msb7k [BITS16] = {0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 1 };
  int msb8k [BITS16] = {0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 0 };
  int msb9k [BITS16] = {0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1 };
 int msb10k [BITS16] = {0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0 };

int freq;
int magnitude, freq1, freq2, phase;
int move = 0;
int stop = 0; 
int slow = 0;
int accel = 0;
int calibrate = 0;
int notmove = 0;

/** Main Function **********************************************/

int main(void)
{
	int	pbClk;
	int i;
	
	//Set D1, D2 as a digital output
	LATD |= 0x0006; TRISD &= 0xFFF9;

	// Set Direction	
	DIR = Forward;

	// Configure the proper PB frequency and the number of wait states
	pbClk = SYSTEMConfigPerformance(SYS_FREQ);
	
	//Set A2,A3 and A14 as a digital output
	LATA |= 0x400C; TRISA &= 0xBFF3;
		
	// Allow vector interrupts
	INTEnableSystemMultiVectoredInt();
	
	mInitAllLEDs();
	
	initInterruptController();
	
	initUART2(pbClk);

	// Configure the proper PB frequency and the number of wait states
	SYSTEMConfigPerformance(SYS_FREQ);

	initADC();
	
	//Set A9 and A10 as a digital input
	TRISAbits.TRISA9=1;
	TRISAbits.TRISA10=1;

	INPUT_A9 = TRUE; 
	INPUT_A10 = TRUE; 

	// zero the freqVector and FFTmag
	
	for (i=0; i<N; i++)
	{
		freqVector[i] = 0;
		singleSidedFFT1[i] = 0;
		singleSidedFFT2[i] = 0;
		phase1[i] = 0;
		phase2[i] = 0;
	}		
	

	while(1) //2575 steps = 1 inch
	{	
		if (calibrate == 1){    //calibrate button from process pressed
			dcalibrate();
			calibrate = 0;
		}
		while (move == TRUE){
			CLK = 0;
			Delayms(speed);
			CLK = 1;	
			Delayms(speed);
			step2++;
			
/*
			if(INPUT_A9 == FALSE){
				step = 0;
				DIR = 1;
					while(step<200) {
					CLK=0;
					Delayms(speed);
					CLK=1;
					Delayms(speed);
					step++;
					}
			}
			if(INPUT_A10 == FALSE){
				step = 0;
				DIR = 0;
					while(step<200) {
					CLK=0;
					Delayms(speed);
					CLK=1;
					Delayms(speed);
					step++;
					}
			}*/
			if(step2 >= 1288){
				step2 = 0;
				//move = FALSE;
				if(data == TRUE){
				    cf++;
				    wave(cf);
				    Delayms(1000);
				    computeFFT();
				    sendDataRS232();
				    notmove == 1;
				    while(notmove == 1){
				    }
			      }
			}
			
		}

	}
CloseOC1();
} //end main


/** Interrupt Handlers *****************************************/
// interrupt the timer 3
void __ISR( _TIMER_3_VECTOR, ipl7) T3Interrupt( void)
{
	int i;
	//LOOP_TIME_PIN = TRUE;
	// When we used the loop time pin to measure the length of this ISR,
	// we measured 400 ns, so you could sample at over 1 MHz.

	sampleBuffer1[sampleIndex].re = ReadADC10(0); // read the ADC into the real part of the samplebuffer
	sampleBuffer1[sampleIndex].im = 0; // the imaginary value is 0.
	
	sampleBuffer2[sampleIndex].re = ReadADC10(2);
	sampleBuffer2[sampleIndex].im = 0;

	// you could shave a little time off this ISR by just zeroing the .im value once, outside the ISR

	// increment the sampleIndex
	if (sampleIndex == (N-1))
	{
		sampleIndex = 0;
	}
	else
	{
		sampleIndex++;
	}	 

	
	//LOOP_TIME_PIN = FALSE;

	// clear interrupt flag and exit
	mT3ClearIntFlag();
} // T3 Interrupt


// UART 2 interrupt handler
// it is set at priority level 2
void __ISR(_UART2_VECTOR, ipl2) IntUart2Handler(void)
{
	char data;
	// Is this an RX interrupt?
	if(mU2RXGetIntFlag())
	{
		// Clear the RX interrupt Flag
		mU2RXClearIntFlag();
	
		data = ReadUART2();
		// Echo what we just received.
		putcUART2(data);
		
		
		switch(data)

		{	
			case 'm':	//move
					move = TRUE;
				break;
			case 'd': 	//take data while moving
					data = TRUE;
				break;
			case 't':	//stop
					move = FALSE;
					data = FALSE;
				break;
			case 'f':	//forward
					DIR = Forward;
				break;
			case 'r':	//reverse
					DIR = Reverse;
				break;
			case 'l':	//slow
					speed++;
				break;
			case 'a':	//accel
					speed--;
				break;
			case 'c':	//calibrate
					calibrate = 1;
				break;
			case 'x': // get data
					cf++;
					wave(cf);
					Delayms(1000);
					computeFFT();
					//Delayms(500);
					sendDataRS232();
					if (cf == 19){
						cf = 0;
						notmove = 0;
						//move == TRUE;
					}
				break;	
		}			

		// Toggle LED to indicate UART activity
		mLED_0_Toggle();
 
	}

	// We don't care about TX interrupt
	if ( mU2TXGetIntFlag() )
	{
		mU2TXClearIntFlag();
	}
}




/** Other Functions ********************************************/
void initADC()
{
	// -------- Set Up ADC --------
	// configure and enable the ADC
	CloseADC10();	// ensure the ADC is off before setting the configuration

	// define setup parameters for OpenADC10
				// Turn module on | output in integer | trigger mode auto | enable  autosample
	#define PARAM1  ADC_MODULE_ON | ADC_FORMAT_INTG | ADC_CLK_AUTO | ADC_AUTO_SAMPLING_ON

	// define setup parameters for OpenADC10
			    // ADC ref external    | disable offset test    | enable scan mode | perform 2 samples | use one buffer | use MUXA mode
	#define PARAM2  ADC_VREF_AVDD_AVSS | ADC_OFFSET_CAL_DISABLE | ADC_SCAN_ON | ADC_SAMPLES_PER_INT_3 | ADC_ALT_BUF_OFF | ADC_ALT_INPUT_OFF

	// define setup parameters for OpenADC10
	// 				  use ADC internal clock | set sample time
	#define PARAM3  ADC_CONV_CLK_INTERNAL_RC | ADC_SAMPLE_TIME_15

	// define setup parameters for OpenADC10
				// set AN4 and AN5
	#define PARAM4	ENABLE_AN3_ANA | ENABLE_AN4_ANA | ENABLE_AN5_ANA

	// define setup parameters for OpenADC10
	// do not assign channels to scan
	//#define PARAM5	SKIP_SCAN_AN0 | SKIP_SCAN_AN1 | SKIP_SCAN_AN2 | SKIP_SCAN_AN3 | SKIP_SCAN_AN6 | SKIP_SCAN_AN7 | SKIP_SCAN_AN8 | SKIP_SCAN_AN9 | SKIP_SCAN_AN10 | SKIP_SCAN_AN11 | SKIP_SCAN_AN12 | SKIP_SCAN_AN13 | SKIP_SCAN_AN14 | SKIP_SCAN_AN15
    #define PARAM5	SKIP_SCAN_AN0 | SKIP_SCAN_AN1 | SKIP_SCAN_AN2 | SKIP_SCAN_AN6 | SKIP_SCAN_AN7 | SKIP_SCAN_AN8 | SKIP_SCAN_AN9 | SKIP_SCAN_AN10 | SKIP_SCAN_AN11 | SKIP_SCAN_AN12 | SKIP_SCAN_AN13 | SKIP_SCAN_AN14 | SKIP_SCAN_AN15

	// use ground as neg ref for A | use AN4 (B4) for input A
	// configure to sample AN4
	SetChanADC10( ADC_CH0_NEG_SAMPLEA_NVREF); // configure to sample AN4
	OpenADC10( PARAM1, PARAM2, PARAM3, PARAM4, PARAM5 ); // configure ADC using parameter define above

	EnableADC10(); // Enable the ADC

	while ( ! mAD1GetIntFlag() ) { } // wait for the first conversion to complete so there will be valid data in ADC result registers
	// ------ DONE SETTING UP ADC ------
}
void initInterruptController(void)
{
	// init Timer3 mode and period (PR3) 
	OpenTimer3( T3_ON | T3_PS_1_1 | T3_SOURCE_INT, samfreq); // produces 100 ms period
															// sampling frequency = 10 kHz
	
	mT3SetIntPriority( 7); 	// set Timer3 Interrupt Priority
	mT3ClearIntFlag(); 		// clear interrupt flag
	mT3IntEnable( 1);		// enable timer3 interrupts
}

void initUART2(int pbClk)
{
	 // define setup Configuration 1 for OpenUARTx
		// Module Enable 
		// Work in IDLE mode 
		// Communication through usual pins 
		// Disable wake-up 
		// Loop back disabled 
		// Input to Capture module from ICx pin 
		// no parity 8 bit 
		// 1 stop bit 
		// IRDA encoder and decoder disabled 
		// CTS and RTS pins are disabled 
		// UxRX idle state is '1' 
		// 16x baud clock - normal speed
	#define config1 	UART_EN | UART_IDLE_CON | UART_RX_TX | UART_DIS_WAKE | UART_DIS_LOOPBACK | UART_DIS_ABAUD | UART_NO_PAR_8BIT | UART_1STOPBIT | UART_IRDA_DIS | UART_DIS_BCLK_CTS_RTS| UART_NORMAL_RX | UART_BRGH_SIXTEEN
	 
	 // define setup Configuration 2 for OpenUARTx
		// IrDA encoded UxTX idle state is '0'
		// Enable UxRX pin
		// Enable UxTX pin
		// Interrupt on transfer of every character to TSR 
		// Interrupt on every char received
		// Disable 9-bit address detect
		// Rx Buffer Over run status bit clear
	 #define config2		UART_TX_PIN_LOW | UART_RX_ENABLE | UART_TX_ENABLE | UART_INT_TX | UART_INT_RX_CHAR | UART_ADR_DETECT_DIS | UART_RX_OVERRUN_CLEAR	
 
	// Open UART2 with config1 and config2
	OpenUART2( config1, config2, pbClk/16/DESIRED_BAUDRATE-1);	// calculate actual BAUD generate value.
 		
	// Configure UART2 RX Interrupt with priority 2
	ConfigIntUART2(UART_INT_PR2 | UART_RX_INT_EN);
}

void sendDataRS232()
{
	int i;
	char RS232_Out_Buffer[128]; // max characters per line (line feed and carriage return count)
	
	sprintf(RS232_Out_Buffer,"\n\rSTART\n\r"); //print START. MATLAB uses this as a start delimiter
	putsUART2(RS232_Out_Buffer);
	
	sprintf(RS232_Out_Buffer,"ROWS=%d\n\r", N); //print the number of rows, so matlab can make the correct sized buffer
	putsUART2(RS232_Out_Buffer);
	
	for(i = 0; i < N; i++)
	{
		sprintf(RS232_Out_Buffer,"%d %d %d %f %f %d %d\n\r",freqVector[i], singleSidedFFT1[i], singleSidedFFT2[i], phase1[i], phase2[i], calcBuffer1[i].re, calcBuffer2[i].re);
		//sprintf(RS232_Out_Buffer,"%d %d %d %f %f\n\r",freqVector[i], singleSidedFFT1[i], singleSidedFFT2[i], phase1[i], phase2[i]);
		putsUART2(RS232_Out_Buffer);
	}
	
	sprintf(RS232_Out_Buffer,"END\n\r"); //output end so matlab knows we're done
	putsUART2(RS232_Out_Buffer);
	
}

void computeFFT()
{
	// when using 256 samples, we measured this function to take about 500 microseconds
	// (not including the time to send rs232 data)
	int i;
	
	// generate frequency vector
	// this is the x-axis of your single sided fft
	for (i=0; i<N/2; i++)
	{
		freqVector[i] = i*(SAMPLEFREQ/2)/((N/2) - 1);
	}
	mT3IntEnable(0); //turns off interrupt while computing FFT
	
	//LOOP_TIME_PIN = TRUE;
	
	for (i=0; i<N; i++)
	{
		if (i<sampleIndex)
		{
			// old chunk
			calcBuffer1[i+(N-sampleIndex)] = sampleBuffer1[i];
			calcBuffer2[i+(N-sampleIndex)] = sampleBuffer2[i];
		}	
		else // i >= sampleIndex
		{
			// new chunk
			calcBuffer1[i-sampleIndex] = sampleBuffer1[i];
			calcBuffer2[i-sampleIndex] = sampleBuffer2[i];
		}	
		
	}	

	// load complex input data into din
	mips_fft16(dout1, calcBuffer1, fftc, scratch1, log2N);
	mips_fft16(dout2, calcBuffer2, fftc, scratch2, log2N);	

	// compute single sided fft
	
//	LOOP_TIME_PIN = FALSE;
	for(i = 0; i < N/2; i++)
	{
		singleSidedFFT1[i] = 2 * ((dout1[i].re*dout1[i].re) + (dout1[i].im*dout1[i].im));
		phase1[i] = atan2(dout1[i].im,dout1[i].re);
		singleSidedFFT2[i] = 2 * ((dout2[i].re*dout2[i].re) + (dout2[i].im*dout2[i].im));
		phase2[i] = atan2(dout2[i].im,dout2[i].re);
	}
	
	computeFFTflag = FALSE;

	// do something with dout
	mT3IntEnable(1); //turn interrupt back on
}


void Delayms( unsigned t)
// This uses Timer 1, can be changed to another timer. Assumes FPB = SYS_FREQ
{
    OpenTimer1(T1_ON | T1_PS_1_256, 0xFFFF);
    while (t--)
    {  // t x 1ms loop
        WriteTimer1(0);
        while (ReadTimer1() < SYS_FREQ/256/1000);
   }
   CloseTimer1();
} // Delayms

//send the AD9833 16 bits
void ad9833_set_freq(int value[]){
	 int i;
	 PIN_A3 = 1; //	output_high(SCLK);
	 PIN_A2 = 0; // output_low(fsync);
	 for (i=0;i<=BITS16-1;i++){
	   PIN_A3 = 1; //output_high(SCLK);
	   Delayms(1);
	     if (value[i] == 0){
	       PIN_A14 = 0; //output_low(SDATA);
	     }
	     else {
	       PIN_A14 = 1; //output_high(SDATA);
	     }
	   Delayms(1); //delay_us(50);
	   PIN_A3 = 0; //output_low(SCLK);
	   Delayms(1); //delay_us(100);
	 }
	 PIN_A3 = 1; //output_high(SCLK);
	 PIN_A2 = 1; //output_high(f);
	 Delayms(3); //delay_us(250);
}

void generateWave(int msb[], int lsb[]){
		 int ad_sine[BITS16] = {0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 }; // control word, set output = sine
		 ad9833_set_freq(ad_sine);
		 ad9833_set_freq(lsb);
		 ad9833_set_freq(msb);
		 int unreset[BITS16] = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
		 ad9833_set_freq(unreset);
}

void wave(int cf){
	if (cf == 1){		
	samfreq = 0xC34F;	//49999
	SAMPLEFREQ = 1600;
	initInterruptController();
	generateWave(msb,lsb100);
	} else if (cf == 2){
	samfreq = 0x61A7;
	SAMPLEFREQ = 3200;
	initInterruptController();
	generateWave(msb,lsb200);
	}else if (cf == 3){
	samfreq = 0x411A;
	SAMPLEFREQ = 4800;
	initInterruptController();
	generateWave(msb,lsb300);
	}else if (cf == 4){
	samfreq = 0x30D3;
	SAMPLEFREQ = 6400;
	initInterruptController();
	generateWave(msb,lsb400);
	}else if (cf == 5){
	samfreq = 0x270F;
	SAMPLEFREQ = 8000;
	initInterruptController();
	generateWave(msb,lsb500);
	}else if (cf == 6){
	samfreq = 0x208D;
	SAMPLEFREQ = 9600;
	initInterruptController();
	generateWave(msb,lsb600);
	}else if (cf == 7){
	samfreq = 0x1BE6;
	SAMPLEFREQ = 11200;
	initInterruptController();
	generateWave(msb,lsb700);
	}else if (cf == 8){
	samfreq = 0x1869;
	SAMPLEFREQ = 12800;
	initInterruptController();
	generateWave(msb,lsb800);
	}else if (cf == 9){
	samfreq = 0x15B3;
	SAMPLEFREQ = 14400;
	initInterruptController();
	generateWave(msb,lsb900);
	}else if (cf == 10){
	samfreq = 0x1387;
	SAMPLEFREQ = 16000;
	initInterruptController();
	generateWave(msb,lsb1k);
	}else if (cf == 11){
	samfreq = 0x09C3;
	SAMPLEFREQ = 32000;
	initInterruptController();
	generateWave(msb2k,lsb2k);
	}else if (cf == 12){
	samfreq = 0x0682;
	SAMPLEFREQ = 48000;	
	initInterruptController();
	generateWave(msb3k,lsb3k);
	}else if (cf == 13){
	samfreq = 0x04E1;
	SAMPLEFREQ = 64000;	
	initInterruptController();
	generateWave(msb4k,lsb4k);
	}else if (cf == 14){
	samfreq = 0x03E7;
	SAMPLEFREQ = 80000;
	initInterruptController();
	generateWave(msb5k,lsb5k);
	}else if (cf == 15){
	samfreq = 0x0341;
	SAMPLEFREQ = 96000;
	initInterruptController();
	generateWave(msb6k,lsb6k);
	}else if (cf == 16){
	samfreq = 0x02CA;
	SAMPLEFREQ = 112000;
	initInterruptController();
	generateWave(msb7k,lsb7k);
	}else if (cf == 17){
	samfreq = 0x0270;
	SAMPLEFREQ = 128000;
	initInterruptController();
	generateWave(msb8k,lsb8k);
	}else if (cf == 18){
	samfreq = 0x022B;
	SAMPLEFREQ = 144000;
	initInterruptController();
	generateWave(msb9k,lsb9k);
	}else if (cf == 19){
	samfreq = 0x01F3;
	SAMPLEFREQ = 160000;
	initInterruptController();
	generateWave(msb10k,lsb10k);
	}
}

void dcalibrate(){
	DIR = Reverse;
	while(INPUT_A10 == TRUE){
		CLK = 0;
		Delayms(speed);
		CLK = 1;	
		Delayms(speed);
	}
		step = 0;
		DIR = Forward;
		while(step<200) {
			CLK=0;
			Delayms(speed);
			CLK=1;
			Delayms(speed);
			step++;
		}
}
